window.Gleam.initSearch([{"doc":"eventsourcing","title":"eventsourcing","content":"# eventsourcing\n\n[![Package Version](https://img.shields.io/hexpm/v/eventsourcing)](https://hex.pm/packages/eventsourcing)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/eventsourcing/)\n\n```sh\ngleam add eventsourcing\n```\n```gleam\nimport eventsourcing\nimport eventsourcing/postgres_store\n\npub type BankAccount {\n  BankAccount(opened: Bool, balance: Float)\n}\n\npub type BankAccountCommand {\n  OpenAccount(account_id: String)\n  DepositMoney(amount: Float)\n  WithDrawMoney(amount: Float)\n}\n\npub type BankAccountEvent {\n  AccountOpened(account_id: String)\n  CustomerDepositedCash(amount: Float, balance: Float)\n  CustomerWithdrewCash(amount: Float, balance: Float)\n}\n\npub fn event_type() {\n  &quot;BankAccountEvent&quot;\n}\n\npub fn encode_event(event: BankAccountEvent) -&gt; String {\n  // ...\n  todo\n}\n\npub fn decode_event(\n  string: String,\n) -&gt; Result(BankAccountEvent, List(dynamic.DecodeError)) {\n  // ...\n  todo\n}\n\npub fn handle(\n  bank_account: BankAccount,\n  command: BankAccountCommand,\n) -&gt; Result(List(BankAccountEvent), Nil) {\n  case command {\n    OpenAccount(account_id) -&gt; Ok([AccountOpened(account_id)])\n    DepositMoney(amount) -&gt; {\n      let balance = bank_account.balance +. amount\n      case amount &gt;. 0.0 {\n        True -&gt; Ok([CustomerDepositedCash(amount:, balance:)])\n        False -&gt; Error(Nil)\n      }\n    }\n    WithDrawMoney(amount) -&gt; {\n      let balance = bank_account.balance -. amount\n      case amount &gt;. 0.0 {\n        True -&gt; Ok([CustomerDepositedCash(amount:, balance:)])\n        False -&gt; Error(Nil)\n      }\n    }\n  }\n}\n\npub fn apply(bank_account: BankAccount, event: BankAccountEvent) {\n  case event {\n    AccountOpened(_) -&gt; BankAccount(..bank_account, opened: True)\n    CustomerDepositedCash(_, balance) -&gt; BankAccount(..bank_account, balance:)\n    CustomerWithdrewCash(_, balance) -&gt; BankAccount(..bank_account, balance:)\n  }\n}\n\npub fn aggregate_type() -&gt; String {\n  &quot;BankAccount&quot;\n}\n\n\npub fn main() {\n  let postgres_store =\n    postgres_store.new(\n      pgo_config: pgo.Config(\n        ..pgo.default_config(),\n        host: &quot;localhost&quot;,\n        database: &quot;postgres&quot;,\n        pool_size: 15,\n        password: option.Some(&quot;password&quot;),\n      ),\n      emtpy_entity: BankAccount(opened: False, balance: 0.0),\n      handle_command_function: handle,\n      apply_function: apply,\n      event_encoder: encode_event,\n      event_decoder: decode_event,\n      event_type: event_type(),\n      event_version: &quot;1&quot;,\n      aggregate_type: aggregate_type(),\n    )\n  let event_sourcing = eventsourcing.new(postgres_store, [])\n  eventsourcing.execute(\n    event_sourcing,\n    &quot;92085b42-032c-4d7a-84de-a86d67123858&quot;,\n    DepositMoney(10.0),\n  )\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/eventsourcing&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"eventsourcing","title":"Aggregate","content":"pub type Aggregate(entity, command, event, error) {\n  Aggregate(\n    entity: entity,\n    handle: Handle(entity, command, event, error),\n    apply: Apply(entity, event),\n  )\n}\n\nAggregate(\n  entity: entity,\n  handle: Handle(entity, command, event, error),\n  apply: Apply(entity, event),\n)\n\n\nSynonyms:\neventsourcing.Aggregate\neventsourcing Aggregate","url":"eventsourcing.html#Aggregate"},{"doc":"eventsourcing","title":"AggregateContext","content":"pub type AggregateContext(entity, command, event, error) {\n  AggregateContext(\n    aggregate_id: AggregateId,\n    aggregate: Aggregate(entity, command, event, error),\n    sequence: Int,\n  )\n}\n\nAggregateContext(\n  aggregate_id: AggregateId,\n  aggregate: Aggregate(entity, command, event, error),\n  sequence: Int,\n)\n\n\nSynonyms:\neventsourcing.AggregateContext\neventsourcing AggregateContext","url":"eventsourcing.html#AggregateContext"},{"doc":"eventsourcing","title":"EventEnvelop","content":"pub type EventEnvelop(event) {\n  MemoryStoreEventEnvelop(\n    aggregate_id: AggregateId,\n    sequence: Int,\n    payload: event,\n  )\n  SerializedEventEnvelop(\n    aggregate_id: AggregateId,\n    sequence: Int,\n    payload: event,\n    event_type: String,\n    event_version: String,\n    aggregate_type: String,\n  )\n}\n\nMemoryStoreEventEnvelop(\n  aggregate_id: AggregateId,\n  sequence: Int,\n  payload: event,\n)\n\n\nSerializedEventEnvelop(\n  aggregate_id: AggregateId,\n  sequence: Int,\n  payload: event,\n  event_type: String,\n  event_version: String,\n  aggregate_type: String,\n)\n\n\nSynonyms:\neventsourcing.EventEnvelop\neventsourcing EventEnvelop","url":"eventsourcing.html#EventEnvelop"},{"doc":"eventsourcing","title":"EventSourcing","content":"pub type EventSourcing(\n  eventstore,\n  entity,\n  command,\n  event,\n  error,\n  aggregatecontext,\n) {\n  EventSourcing(\n    event_store: EventStore(\n      eventstore,\n      entity,\n      command,\n      event,\n      error,\n    ),\n    queries: List(Query(event)),\n  )\n}\n\nEventSourcing(\n  event_store: EventStore(\n    eventstore,\n    entity,\n    command,\n    event,\n    error,\n  ),\n  queries: List(Query(event)),\n)\n\n\nSynonyms:\neventsourcing.EventSourcing\neventsourcing EventSourcing","url":"eventsourcing.html#EventSourcing"},{"doc":"eventsourcing","title":"EventStore","content":"pub type EventStore(eventstore, entity, command, event, error) {\n  EventStore(\n    eventstore: eventstore,\n    load_aggregate: fn(eventstore, AggregateId) -&gt;\n      AggregateContext(entity, command, event, error),\n    commit: fn(\n      eventstore,\n      AggregateContext(entity, command, event, error),\n      List(event),\n    ) -&gt;\n      List(EventEnvelop(event)),\n  )\n}\n\nEventStore(\n  eventstore: eventstore,\n  load_aggregate: fn(eventstore, AggregateId) -&gt;\n    AggregateContext(entity, command, event, error),\n  commit: fn(\n    eventstore,\n    AggregateContext(entity, command, event, error),\n    List(event),\n  ) -&gt;\n    List(EventEnvelop(event)),\n)\n\n\nSynonyms:\neventsourcing.EventStore\neventsourcing EventStore","url":"eventsourcing.html#EventStore"},{"doc":"eventsourcing","title":"execute","content":"pub fn execute(\n  event_sourcing: EventSourcing(a, b, c, d, e, f),\n  aggregate_id aggregate_id: String,\n  command command: c,\n) -&gt; Result(Nil, e)\n\nSynonyms:\neventsourcing.execute\neventsourcing execute","url":"eventsourcing.html#execute"},{"doc":"eventsourcing","title":"new","content":"pub fn new(\n  event_store: EventStore(a, b, c, d, e),\n  queries: List(fn(String, List(EventEnvelop(d))) -&gt; Nil),\n) -&gt; EventSourcing(a, b, c, d, e, f)\n\nSynonyms:\neventsourcing.new\neventsourcing new","url":"eventsourcing.html#new"},{"doc":"eventsourcing","title":"eventsourcing","content":"","url":"eventsourcing.html"},{"doc":"eventsourcing/memory_store","title":"MemoryStore","content":"pub opaque type MemoryStore(entity, command, event, error)\n\n\nSynonyms:\neventsourcing/memory_store.MemoryStore\neventsourcing/memory_store MemoryStore","url":"eventsourcing/memory_store.html#MemoryStore"},{"doc":"eventsourcing/memory_store","title":"load_aggregate","content":"pub fn load_aggregate(\n  memory_store: MemoryStore(a, b, c, d),\n  aggregate_id: String,\n) -&gt; AggregateContext(a, b, c, d)\n\nSynonyms:\neventsourcing/memory_store.load_aggregate\neventsourcing/memory_store load_aggregate","url":"eventsourcing/memory_store.html#load_aggregate"},{"doc":"eventsourcing/memory_store","title":"load_events","content":"pub fn load_events(\n  memory_store: MemoryStore(a, b, c, d),\n  aggregate_id: String,\n) -&gt; List(EventEnvelop(c))\n\nSynonyms:\neventsourcing/memory_store.load_events\neventsourcing/memory_store load_events","url":"eventsourcing/memory_store.html#load_events"},{"doc":"eventsourcing/memory_store","title":"new","content":"pub fn new(\n  emtpy_entity empty_entity: a,\n  handle_command_function handle: fn(a, b) -&gt; Result(List(c), d),\n  apply_function apply: fn(a, c) -&gt; a,\n) -&gt; EventStore(MemoryStore(a, b, c, d), a, b, c, d)\n\nSynonyms:\neventsourcing/memory_store.new\neventsourcing/memory_store new","url":"eventsourcing/memory_store.html#new"},{"doc":"eventsourcing/memory_store","title":"eventsourcing/memory_store","content":"","url":"eventsourcing/memory_store.html"},{"doc":"eventsourcing/postgres_store","title":"PostgresStore","content":"pub opaque type PostgresStore(entity, command, event, error)\n\n\nSynonyms:\neventsourcing/postgres_store.PostgresStore\neventsourcing/postgres_store PostgresStore","url":"eventsourcing/postgres_store.html#PostgresStore"},{"doc":"eventsourcing/postgres_store","title":"load_aggregate","content":"pub fn load_aggregate(\n  postgres_store: PostgresStore(a, b, c, d),\n  aggregate_id: String,\n) -&gt; AggregateContext(a, b, c, d)\n\nSynonyms:\neventsourcing/postgres_store.load_aggregate\neventsourcing/postgres_store load_aggregate","url":"eventsourcing/postgres_store.html#load_aggregate"},{"doc":"eventsourcing/postgres_store","title":"load_events","content":"pub fn load_events(\n  postgres_store: PostgresStore(a, b, c, d),\n  aggregate_id: String,\n) -&gt; Result(List(EventEnvelop(c)), QueryError)\n\nSynonyms:\neventsourcing/postgres_store.load_events\neventsourcing/postgres_store load_events","url":"eventsourcing/postgres_store.html#load_events"},{"doc":"eventsourcing/postgres_store","title":"new","content":"pub fn new(\n  pgo_config pgo_config: Config,\n  emtpy_entity empty_entity: a,\n  handle_command_function handle: fn(a, b) -&gt; Result(List(c), d),\n  apply_function apply: fn(a, c) -&gt; a,\n  event_encoder event_encoder: fn(c) -&gt; String,\n  event_decoder event_decoder: fn(String) -&gt;\n    Result(c, List(DecodeError)),\n  event_type event_type: String,\n  event_version event_version: String,\n  aggregate_type aggregate_type: String,\n) -&gt; EventStore(PostgresStore(a, b, c, d), a, b, c, d)\n\nSynonyms:\neventsourcing/postgres_store.new\neventsourcing/postgres_store new","url":"eventsourcing/postgres_store.html#new"},{"doc":"eventsourcing/postgres_store","title":"eventsourcing/postgres_store","content":"","url":"eventsourcing/postgres_store.html"}]);